package kernel.peripherals;

import kernel.log.Log;
import lib.Debug;
import cc.Peripheral;
import lib.Tags;
import lib.Item;
import cc.periphs.ItemStorage.ReducedItemInfo;
import lua.Table;

using Lambda;

class ItemInfo {
	public final count:Int;
	public final name:Item;
	public final nbt:Null<String>;

	@:allow(kernel.peripherals.Inventory)
	private function new(from:ReducedItemInfo) {
		this.count = from.count;
		this.name = from.name;
		this.nbt = from.nbt;
	}
}

class DetailedItemInfo extends ItemInfo {
	public final displayName:String;
	public final maxCount:Int;
	public final tags:Tags;
	public final durability:Null<Float>;
	public final damage:Null<Int>;
	public final maxDamage:Null<Int>;
	public final enchantments:Array<{displayName:String, level:Int, name:String}>;

	@:allow(kernel.peripherals.Inventory)
	private function new(from:cc.periphs.ItemStorage.DetailedItemInfo) {
		super(from);

		this.tags = Tags.fromBoolTable(from.tags);

		this.displayName = from.displayName;
		this.maxCount = from.maxCount;
		this.durability = from.durability;
		this.damage = from.damage;
		this.maxDamage = from.maxDamage;

		if (from.enchantments != null) {
			this.enchantments = from.enchantments;
		} else {
			this.enchantments = [];
		}
	}
}

class Inventory implements IPeripheral {
	public static inline final TYPE_NAME:String = "inventory";

	private final addr:String;

	public function new(addr:String) {
		this.addr = addr;
	}

	public function getAddr():String {
		return this.addr;
	}

	public function getType():String {
		return TYPE_NAME;
	}

	/**
		Get the size of this inventory.
	**/
	public function size():Int {
		return Peripheral.call(this.addr, "size");
	}

	/**
		List all items in this inventory.
	**/
	public function list():Map<Int, ItemInfo> {
		var list:Map<Int, ReducedItemInfo> = Table.toMap(Peripheral.call(this.addr, "list"));
		var rtn:Map<Int, ItemInfo> = new Map();

		for (k => v in list) {
			rtn.set(k - 1, new ItemInfo(v));
		}

		return rtn;
	}

	/**
		Get detailed information about an item.
	**/
	public function getItemDetail(slot:Int):DetailedItemInfo {
		return new DetailedItemInfo(Peripheral.call(this.addr, "getItemDetail", slot + 1));
	}

	/**
		Get the maximum number of items which can be stored in this slot.
		Typically this will be limited to 64 items. 
		However, some inventorwies (such as barrels or caches) can store hundreds or thousands of items in one slot.
		Keep in mind that this does not depend on that item stored in this slot.
	**/
	public function getItemLimit(slot:Int):Int {
		return Peripheral.call(this.addr, "getItemLimit", slot + 1);
	}

	/**
		Push items from one inventory to another connected one. Needs to be on the same wired network.
	**/
	public function pushItems(to:String, fromSlot:Int, ?limit:Int, ?toSlot:Int):Int {
		return Peripheral.call(this.addr, "pushItems", to, fromSlot + 1, limit, toSlot);
	}

	/**
		Pull items from a connected inventory into this one. Needs to be on the same wired network.
	**/
	public function pullItems(from:String, fromSlot:Int, ?limit:Int, ?toSlot:Int):Int {
		return Peripheral.call(this.addr, "pullItems", from, fromSlot + 1, limit, toSlot);
	}
}
